import json
import threading
import logging
import hashlib
import datetime
import pytz
import requests
import os


def synchronized(method):

    outer_lock = threading.Lock()
    lock_name = "__"+method.__name__+"_lock"+"__"

    def sync_method(self, *args, **kws):
        with outer_lock:
            if not hasattr(self, lock_name): setattr(self, lock_name, threading.Lock())
            lock = getattr(self, lock_name)
            with lock:
                return method(self, *args, **kws)

    return sync_method

class Time_Utils(object):

    # TODO: tidy this up
    @staticmethod
    def current_epoch():
        utc_date_time = datetime.datetime.utcnow()
        utc_date_time = utc_date_time.replace(tzinfo=pytz.utc)
        return utc_date_time.isoformat()

class Crypto(object):

    @staticmethod
    def sha256(file_path):
        file_pointer = open(file_path, 'rb')
        try:
            sha256_hash = hashlib.sha256()
            sha256_hash.update(file_pointer.read())
            return sha256_hash.hexdigest()

        except Exception, e:
            logging.exception("Error hashing file %s: %s", file_path, e)
            raise

        finally:
            file_pointer.close()


class URL(object):

    @staticmethod
    def join(*args):
        return "/".join(map(lambda x: str(x).rstrip('/'), args))


class Dict_Diff(object):
    def __init__(self, current_dict, latest_dict):
        self.latest_dict, self.current_dict = latest_dict, current_dict
        self.latest_keys, self.current_keys = [
            set(d.keys()) for d in (latest_dict, current_dict)
        ]
        self.intersect = self.latest_keys.intersection(self.current_keys)

    def added_keys(self):
        return self.latest_keys - self.intersect

    def added_values(self):
        return set(self.latest_dict[key] for key in self.added_keys()
                   if key in self.latest_dict)

    def removed_keys(self):
        return self.current_keys - self.intersect

    def removed_values(self):
        return set(self.current_dict[key] for key in self.removed_keys()
                   if key in self.current_dict)

    def changed_keys(self):
        return set(o for o in self.intersect
                   if self.current_dict[o] != self.latest_dict[o])

    def changed_values(self):
        return set(self.latest_dict[key] for key in self.changed_keys()
                   if key in self.latest_dict)

    def unchanged_keys(self):
        return set(o for o in self.intersect
                   if self.__values_match(o))

    def unchanged_values(self):
        return set(self.current_dict[key] for key in self.unchanged_keys()
                   if key in self.current_dict)

    def __values_match(self, key):
        same = (self.current_dict[key] == self.latest_dict[key])
        return same

class Files(object):

    @staticmethod
    def download(src, dest):
        req = requests.get(src)
        with open(dest, "wb") as dest_file:
            dest_file.write(req.content)

    @staticmethod
    def set_permissions(file_path, permissions):
        os.chmod(file_path, permissions)