import logging
from apscheduler.events import JobStoreEvent


class Job_Manager(object):

    def __init__(self, scheduler):
        self.scheduler = scheduler

        self.__initializeScheduler()

    def add_job(self, job):
        args = [job.args] if job.args is not None else None
        self.scheduler.add_interval_job(job.task, seconds=job.interval,
                                        args=args,  name=job.name,
                                        max_instances=1)

    def remove_job(self, name):
        matchedJobs = self.__get_jobs(name)
        self.__remove_jobs(matchedJobs)

    def get_jobs(self):
        return self.scheduler.get_jobs()

    def __listener(self, event):
        if isinstance(event, JobStoreEvent):
            logging.info("Job added")
        elif event.exception:
            logging.info('The %s job failed' % event)

    def __get_jobs(self, name):
        return [job for job in self.get_jobs() if job.name == name]

    def __remove_jobs(self, matchedJobs):
        for job in matchedJobs:
            self.__removeJob(job)

    def __removeJob(self, job):
        self.scheduler.unschedule_job(job)

    def __initializeScheduler(self):
        if self.scheduler is not None:
            self.scheduler.add_listener(self.__listener)
            self.scheduler.start()