import logging
import os
from helpers import Crypto, Files, URL


class Plugin_Cache_Manager(object):


    def __init__(self, context):
        self._context = context

    def get_cache_location(self, plugin_name):
        cache_location = self._context.plugin_cache
        return os.path.join(cache_location, plugin_name)

    def load(self, plugin):
        try:
            self.__load_cached_plugin(plugin)
        except Exception, e:
            logging.exception("failed to load plugin %s: %s", plugin.name, e)
            raise

    def __load_cached_plugin(self, plugin):
        if not self.is_plugin_cached(plugin):
            self.__cache_plugin(plugin)

    def is_plugin_cached(self, plugin):
        return (os.path.isfile(plugin.location) and
                self.__original_plugin(plugin))

    def __original_plugin(self, plugin):
        return Crypto.sha256(plugin.location) == plugin.sha256

    def __cache_plugin(self, plugin):
        self.__remove_from_cache(plugin)
        self.__cache(plugin)

    def __remove_from_cache(self, plugin):
        try:
            os.remove(plugin.location)
        except Exception, e:
            # This is safe to continue, probably no previous entry
            logging.debug("Failed to delete cache entry: %s", e)

    def __cache(self, plugin):
        plugin_source = self.__get_plugin_source(plugin)
        self.__download_exec_file(plugin_source, plugin.location)
        logging.info("Cached plugin: %s", plugin.name)

    def __get_plugin_source(self, plugin):
        return URL.join(self._context.plugin_api, plugin.name, 'content')

    def __download_exec_file(self, source, destination):
        try:
            Files.download(source, destination)
            Files.set_permissions(destination, 0755)
        except Exception, e:
            logging.debug("Error downloading plugin: %s", e)
            raise