import logging
import re as regex
from helpers import Time_Utils


class Plugin_Response_Handler(object):

    def __init__(self, context, results_publisher):
        self._context = context
        self._publisher = results_publisher

    def handle_response(self, plugin, response):
        result = response[0]
        data = response[1]
        try:
            self.__publish_metrics(plugin.name, data)
        except Exception, e:
            logging.exception("Error publishing results for %s: %s",
                              plugin.name, e)

    def __publish_result(self, plugin, result):
        message = self.__generate_result_message(plugin.name, result)
        self._publisher.publish_result(message)

    def __generate_result_message(self, name, result):
        return "[{ \"id\" : \"%s\", \"return_code\" : %i }]" % (name, result)

    def __publish_metrics(self, plugin_name, data):
        metrics = self.__get_metrics(data)
        self.__publish_processed_metrics(plugin_name, metrics)

    def __get_metrics(self, data):
        metrics = {}
        if self.__contains_metrics(data):
            metrics = self.__process_metrics(data)

        return metrics

    def __contains_metrics(self, data):
        return '|' in data

    # TODO: tidy this
    def __process_metrics(self, data):
        # We only need the info after the | character
        perf_data = data.split('|')[1]
        perf_values = perf_data.split(' ')

        # For v1 we just want key pairs with ='s without the thresholds
        perf_dict = {}
        for position, item in enumerate(perf_values):
            if '=' in item:
                perf_items_list = perf_values[position].split('=')

                # Format the perf data so that it doesnt blow up the app
                perf_value_name = regex.sub(' ', '', perf_items_list[0])
                perf_value_name = regex.sub('\.', '_', perf_value_name)
                perf_value_name = regex.sub('\'', '', perf_value_name)
                perf_value_name = regex.sub('/', 'root', perf_value_name)
                perf_value_name = perf_value_name.lower()

                # Throw away everything after ; and only keep the numbers
                perf_value_metric = perf_items_list[1].split(';')[0]
                perf_value_metric = regex.sub(r'[^\d.]+', '', perf_value_metric)

                # Finally, create the dictionary
                perf_dict[perf_value_name] = perf_value_metric

        return perf_dict

    def __publish_processed_metrics(self, plugin_name, metrics):
        for metric in metrics:
            message = self.__generate_metric_message(plugin_name,
                                                     metric,
                                                     metrics[metric])
            self._publisher.publish_metric(message)

    def __generate_metric_message(self, plugin_name, metric_name, metric_value):
        tenant = self._context.tenant
        agent = self._context.node_name
        metric_path = "%s.%s.%s.%s" % (tenant, agent, plugin_name, metric_name)
        timestamp = Time_Utils.current_epoch()

        return "%s %s %s" % (metric_path, metric_value, timestamp)
